// ignore_for_file: camel_case_types, constant_identifier_names, non_constant_identifier_names, unnecessary_this
import './Seq.dart' as seq_mod;
import './Types.dart' as types_mod;

types_mod.Tuple2<T, T>? Function(T) makeRangeStepFunction<T>(T step, T stop, T zero, T Function(T, T) add, int Function(T, T) compare) {
    final stepComparedWithZero = compare(step, zero);
    if (stepComparedWithZero == 0) {
        throw Exception('The step of a range cannot be zero');
    }
    final stepGreaterThanZero = stepComparedWithZero > 0;
    return (T x) {
        final comparedWithLast = compare(x, stop);
        late final bool tmp_1;
        if (stepGreaterThanZero && (comparedWithLast <= 0)) {
            tmp_1 = true;
        } else if (!stepGreaterThanZero) {
            tmp_1 = comparedWithLast >= 0;
        } else {
            tmp_1 = false;
        }
        if (tmp_1) {
            return types_mod.Tuple2(x, add(x, step));
        } else {
            return null;
        }
    };
}

Iterable<T> integralRangeStep<T>(T start, T step, T stop, T zero, T Function(T, T) add, int Function(T, T) compare) {
    final stepFn = makeRangeStepFunction<T>(step, stop, zero, add, compare);
    return seq_mod.delay<T>(() => seq_mod.unfold<T, T>(stepFn, start));
}

Iterable<double> rangeDouble(double start, double step, double stop) => integralRangeStep<double>(start, step, stop, 0.0, (double x, double y) => x + y, (double e1, double e2) => e1.compareTo(e2));

Iterable<int> rangeInt(int start, int step, int stop) => integralRangeStep<int>(start, step, stop, 0, (int x, int y) => x + y, (int e1, int e2) => e1.compareTo(e2));

Iterable<int> rangeChar(int start, int stop) {
    types_mod.Tuple2<int, int>? stepFn(int c) {
        if (c <= stop) {
            return types_mod.Tuple2(c, c + 1);
        } else {
            return null;
        }
    }
    return seq_mod.delay<int>(() => seq_mod.unfold<int, int>(stepFn, start));
}

